% Resiudal function containing the projection conditions

function res=projections_res(dr_coeff0);


global N beta alpha delta sigma rho phi A gamma tau ord psi Le b chi mupar
global kmin kmax amin amax 
global nodes xgrid REG_mat B0_MON B1_MON B2_MON RESIDUALS weights_vec len_nodes
global k1_mat k2_mat K_mat
global a1_mat a2_mat
global ap1_mat ap2_mat 
global cheby_ap1_mat cheby_ap2_mat
global regvec_labor1 regvec_labor2  mumin mumax


% decompose coefficient vector
dr_k1=dr_coeff0(0*end/3+1:1*end/3);
dr_k2=dr_coeff0(1*end/3+1:2*end/3);
dr_mu=dr_coeff0(2*end/3+1:3*end/3);

kp1_mat=REG_mat*dr_k1;
kp2_mat=REG_mat*dr_k2;
Kp_mat=kp1_mat+kp2_mat;

cac1_mat=phi/2*k1_mat.*(kp1_mat./k1_mat-1).^2;
cac2_mat=phi/2*k2_mat.*(kp2_mat./k2_mat-1).^2;
CAC_mat=cac1_mat+cac2_mat;

mu_mat=exp(REG_mat*dr_mu);

REG_MU=chebpol(8,scaldown(mu_mat,mumin,mumax));

REG_A1=chebpol(8,scaldown(a1_mat,amin,amax));
REG_A2=chebpol(8,scaldown(a2_mat,amin,amax));

REG_K1=chebpol(8,scaldown(k1_mat,kmin,kmax));
REG_K2=chebpol(8,scaldown(k2_mat,kmin,kmax));

REGMAT1=makepoly_MUAK(REG_MU,REG_A1,REG_K1);
REGMAT2=makepoly_MUAK(REG_MU,REG_A2,REG_K2);

l1_mat=REGMAT1*regvec_labor1;
l2_mat=REGMAT2*regvec_labor2;

c1_mat=[[b(1)*(Le-l1_mat).^(-1/chi(1))]./[a1_mat.*(1-alpha).*A.*(l1_mat.^(mupar(1)-1)).*...
    (alpha.*k1_mat.^(mupar(1))+(1-alpha)*l1_mat.^mupar(1)).^(1/mupar(1)-1)]].^(-chi(1));
c2_mat=[[b(2)*(Le-l2_mat).^(-1/chi(2))]./[a2_mat.*(1-alpha).*A.*(l2_mat.^(mupar(2)-1)).*...
    (alpha.*k2_mat.^(mupar(2))+(1-alpha)*l2_mat.^mupar(2)).^(1/mupar(2)-1)]].^(-chi(2));
C_mat=c1_mat+c2_mat;

mu1_mat=[tau(1).*c1_mat.^(-1/chi(1)).*[[c1_mat.^(1-1/chi(1))+b(1).*(Le-l1_mat).^(1-1/chi(1))].^[(1-1/gamma(1))./(1-1/chi(1))-1]]];
mu2_mat=[tau(2).*c2_mat.^(-1/chi(2)).*[[c2_mat.^(1-1/chi(2))+b(2).*(Le-l2_mat).^(1-1/chi(2))].^[(1-1/gamma(2))./(1-1/chi(2))-1]]];

% compute output levels
y1_mat=A.*a1_mat.*[alpha.*k1_mat.^mupar(1)+(1-alpha).*l1_mat.^mupar(1)].^(1/mupar(1));
y2_mat=A.*a2_mat.*[alpha.*k2_mat.^mupar(2)+(1-alpha).*l2_mat.^mupar(2)].^(1/mupar(2));
Y_mat=y1_mat+y2_mat;

% compute chebyshev polynomials from next period's endogenous
% states
cheby_kp1_mat=chebpol_2(ord,2*[kp1_mat-kmin]/[kmax-kmin]-1);
cheby_kp2_mat=chebpol_2(ord,2*[kp2_mat-kmin]/[kmax-kmin]-1);

REGp=makepoly_2_4(cheby_kp1_mat,cheby_kp2_mat,cheby_ap1_mat,cheby_ap2_mat);

kpp1_mat=REGp*dr_k1;
kpp2_mat=REGp*dr_k2;
Kpp_mat=kpp1_mat+kpp2_mat;

cacp1_mat=phi/2*kp1_mat.*(kpp1_mat./kp1_mat-1).^2;
cacp2_mat=phi/2*kp2_mat.*(kpp2_mat./kp2_mat-1).^2;
CACp_mat=cacp1_mat+cacp2_mat;

mup_mat=exp(REGp*dr_mu);

REG_MUp=chebpol(8,scaldown(mup_mat,mumin,mumax));

REG_Ap1=chebpol(8,scaldown(ap1_mat,amin,amax));
REG_Ap2=chebpol(8,scaldown(ap2_mat,amin,amax));

REG_Kp1=chebpol(8,scaldown(kp1_mat,kmin,kmax));
REG_Kp2=chebpol(8,scaldown(kp2_mat,kmin,kmax));

REGMATp1=makepoly_MUAK(REG_MUp,REG_Ap1,REG_Kp1);
REGMATp2=makepoly_MUAK(REG_MUp,REG_Ap2,REG_Kp2);

lp1_mat=REGMATp1*regvec_labor1;
lp2_mat=REGMATp2*regvec_labor2;

cp1_mat=[[b(1)*(Le-lp1_mat).^(-1/chi(1))]./[ap1_mat.*(1-alpha).*A.*(lp1_mat.^(mupar(1)-1)).*...
    (alpha.*kp1_mat.^(mupar(1))+(1-alpha)*lp1_mat.^mupar(1)).^(1/mupar(1)-1)]].^(-chi(1));
cp2_mat=[[b(2)*(Le-lp2_mat).^(-1/chi(2))]./[ap2_mat.*(1-alpha).*A.*(lp2_mat.^(mupar(2)-1)).*...
    (alpha.*kp2_mat.^(mupar(2))+(1-alpha)*lp2_mat.^mupar(2)).^(1/mupar(2)-1)]].^(-chi(2));
Cp_mat=cp1_mat+cp2_mat;

mup1_mat=[tau(1).*cp1_mat.^(-1/chi(1)).*[cp1_mat.^(1-1/chi(1))+b(1).*(Le-lp1_mat).^(1-1/chi(1))].^[(1-1/gamma(1))./(1-1/chi(1))-1]];
mup2_mat=[tau(2).*cp2_mat.^(-1/chi(2)).*[cp2_mat.^(1-1/chi(2))+b(2).*(Le-lp2_mat).^(1-1/chi(2))].^[(1-1/gamma(2))./(1-1/chi(2))-1]];

% compute output levels
yp1_mat=A.*ap1_mat.*[alpha.*kp1_mat.^mupar(1)+(1-alpha).*lp1_mat.^mupar(1)].^(1/mupar(1));
yp2_mat=A.*ap2_mat.*[alpha.*kp2_mat.^mupar(2)+(1-alpha).*lp2_mat.^mupar(2)].^(1/mupar(2));
Yp_mat=yp1_mat+yp2_mat;

NN=size(REGp,1);
NN2=NN/len_nodes;

LHS1=mu1_mat(1:NN2).*(1+phi.*(kp1_mat(1:NN2)./k1_mat(1:NN2)-1));   
LHS2=mu2_mat(1:NN2).*(1+phi.*(kp2_mat(1:NN2)./k2_mat(1:NN2)-1)); 
LHS3=Y_mat(1:NN2);


        
RHS1=reshape(beta*mup1_mat.*...
    [[1+phi*(1+1/2.*(kpp1_mat./kp1_mat-1)).*(kpp1_mat./kp1_mat-1)]+...
    A*ap1_mat.*alpha.*kp1_mat.^(mupar(1)-1).*[alpha.*kp1_mat.^(mupar(1))+(1-alpha).*lp1_mat.^mupar(1)].^(1./mupar(1)-1)],NN2,len_nodes)*weights_vec;
RHS2=reshape(beta*mup2_mat.*...
    [[1+phi*(1+1/2.*(kpp2_mat./kp2_mat-1)).*(kpp2_mat./kp2_mat-1)]+...
    A*ap2_mat.*alpha.*kp2_mat.^(mupar(2)-1).*[alpha.*kp2_mat.^(mupar(2))+(1-alpha).*lp2_mat.^mupar(2)].^(1./mupar(2)-1)],NN2,len_nodes)*weights_vec;
RHS3=C_mat(1:NN2)+[Kp_mat(1:NN2)-K_mat(1:NN2)]+CAC_mat(1:NN2);


RES1=[LHS1-RHS1]./LHS1;
RES2=[LHS2-RHS2]./LHS2;
RES3=[LHS3-RHS3]./LHS3;

RESIDUALS=[RES1 RES2 RES3]; 

REG=REG_mat(1:NN2,:);
weights=[B0_MON;ones(4*N,1)*B1_MON;ones(length(RES1)-4*N-1,1)*B2_MON];
res=[REG'*(RES1.*weights);REG'*(RES2.*weights);REG'*(RES3.*weights)];
%disp(mean(mean(abs(RESIDUALS))));