
clear all

cpu0=cputime;

global N beta alpha delta sigma rho phi A gamma tau ord psi Le chi b mupar
global kmin kmax amin amax cmin cmax
global nodes xgrid REG_mat B0_MON B1_MON B2_MON RESIDUALS
global k1_mat k2_mat k3_mat k4_mat k5_mat k6_mat K_mat
global a1_mat a2_mat a3_mat a4_mat a5_mat a6_mat
global ap1_mat ap2_mat ap3_mat ap4_mat ap5_mat ap6_mat
global y1_mat y2_mat y3_mat y4_mat y5_mat y6_mat Y_mat
global cheby_ap1_mat cheby_ap2_mat cheby_ap3_mat
global cheby_ap4_mat cheby_ap5_mat cheby_ap6_mat
global regvec_labor1 regvec_labor2 regvec_labor3 regvec_labor4 regvec_labor5 regvec_labor6 mumin mumax

%%%%%%%%%%%%%%%%%%%%%%
% 1. Getting started %
%%%%%%%%%%%%%%%%%%%%%%

approx=1;   % set order of approximation for perturbation solution (initilization)
ord=2;      % set order of approximation for monomial Galerkin method
kmin=0.8;  % Set bounds for the state space
kmax=1.2;
amin=exp(-0.16);
amax=exp(+0.16);

% 1) Calibration
N=6;
beta=0.99;
alpha=0.36;
delta=0.025;
sigma=0.01;
rho=0.95;
phi=0.5;
A=(1-beta)/(alpha*beta);
gammalow=0.2;
gammahigh=0.4;
gamma=[gammalow:(gammahigh-gammalow)/(N-1):gammahigh];
chilow=0.75;
chihigh=0.9;
chi=[chilow:(chihigh-chilow)/(N-1):chihigh];
muparlow=-0.3;
muparhigh=0.3;
mupar=[muparlow:(muparhigh-muparlow)/(N-1):muparhigh];
Le=2.5;


% 2) Compute the steady state, the Negishi weights and the labor weight in
% the utility function

% a)
kss=1;
lss=1;
Kss=N*kss;
Lss=N*lss;
css=A*kss^alpha*lss^(1-alpha);
Css=N*css;
lnass=0;
ass=1;
thetass=1;
Yss=N*A;

b=zeros(1,N);
for j=1:N
    b(j)=(1-alpha)*A^(1-1/chi(j))/[(Le-lss)^(-1/chi(j))];
end


% b) Compute Negishi weights
tau=zeros(1,N);
for j=1:N
    tau(j)=[css^(-1/chi(j))*[css^(1-1/chi(j))+b(j)*(Le-lss)^(1-1/chi(j))]^[(1-1/gamma(j))/(1-1/chi(j))-1]]^(-1);
end
muss=1;




% 4) Compute the steady state in logs
lnkss=log(kss);
lnKss=log(Kss);
lncss=log(css);
lnCss=log(Css);
lnthetass=log(thetass);
lnYss=log(Yss);
lnLss=log(Lss);
lnlss=log(lss);
lnmuss=log(muss);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2. Construct first-order approximation using method by Klein (2000)  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% 1) Define symbolic variables
syms c1 lna1 k1 theta1 cp1 lnap1 kp1 thetap1 l1 lp1
syms c2 lna2 k2 theta2 cp2 lnap2 kp2 thetap2 l2 lp2
syms c3 lna3 k3 theta3 cp3 lnap3 kp3 thetap3 l3 lp3
syms c4 lna4 k4 theta4 cp4 lnap4 kp4 thetap4 l4 lp4
syms c5 lna5 k5 theta5 cp5 lnap5 kp5 thetap5 l5 lp5
syms c6 lna6 k6 theta6 cp6 lnap6 kp6 thetap6 l6 lp6 
syms C Cp K Kp Y Yp CAC CACp mu mup L Lp

% 1) Define symbolic variables
syms lnc1 lna1 lnk1 lntheta1 lncp1 lnap1 lnkp1 lnthetap1 lnl1 lnlp1
syms lnc2 lna2 lnk2 lntheta2 lncp2 lnap2 lnkp2 lnthetap2 lnl2 lnlp2
syms lnc3 lna3 lnk3 lntheta3 lncp3 lnap3 lnkp3 lnthetap3 lnl3 lnlp3
syms lnc4 lna4 lnk4 lntheta4 lncp4 lnap4 lnkp4 lnthetap4 lnl4 lnlp4
syms lnc5 lna5 lnk5 lntheta5 lncp5 lnap5 lnkp5 lnthetap5 lnl5 lnlp5 
syms lnc6 lna6 lnk6 lntheta6 lncp6 lnap6 lnkp6 lnthetap6 lnl6 lnlp6
syms lnC lnCp lnK lnKp lnY lnYp CAC CACp lnmu lnmup lnL lnLp

% 2) Construct the system of equilibrium conditions

f1=exp(lnmu)*[1+phi*(exp(lnkp1)/exp(lnk1)-1)]-...
    beta*exp(lnmup)*[exp(lnthetap1)+exp(lnap1)*alpha*A*...
    exp(lnkp1)^(mupar(1)-1)*[alpha*exp(lnkp1)^(mupar(1))+(1-alpha)*exp(lnlp1)^(mupar(1))]^(1/mupar(1)-1)];
f1b=b(1)*exp(lnc1)^(1/chi(1))*(Le-exp(lnl1))^(-1/chi(1))-...
    exp(lna1)*(1-alpha)*A*exp(lnl1)^(mupar(1)-1)*[alpha*exp(lnk1)^(mupar(1))+(1-alpha)*exp(lnl1)^(mupar(1))]^(1/mupar(1)-1);
f2=exp(lntheta1)-[1+phi*[1+0.5*(exp(lnkp1)/exp(lnk1)-1)]*(exp(lnkp1)/exp(lnk1)-1)];
f3=tau(1)*[exp(lnc1)^(-1/chi(1))*[exp(lnc1)^(1-1/chi(1))+b(1)*[Le-exp(lnl1)]^(1-1/chi(1))]^[(1-1/gamma(1))/(1-1/chi(1))-1]]-exp(lnmu);
f4=lnap1-rho*lna1;

f5=exp(lnmu)*[1+phi*(exp(lnkp2)/exp(lnk2)-1)]-...
    beta*exp(lnmup)*[exp(lnthetap2)+exp(lnap2)*alpha*A*...
    exp(lnkp2)^(mupar(2)-1)*[alpha*exp(lnkp2)^(mupar(2))+(1-alpha)*exp(lnlp2)^(mupar(2))]^(1/mupar(2)-1)];
f5b=b(2)*exp(lnc2)^(1/chi(2))*(Le-exp(lnl2))^(-1/chi(2))-...
    exp(lna2)*(1-alpha)*A*exp(lnl2)^(mupar(2)-1)*[alpha*exp(lnk2)^(mupar(2))+(1-alpha)*exp(lnl2)^(mupar(2))]^(1/mupar(2)-1);
f6=exp(lntheta2)-[1+phi*[1+0.5*(exp(lnkp2)/exp(lnk2)-1)]*(exp(lnkp2)/exp(lnk2)-1)];
f7=[tau(2)*exp(lnc2)^(-1/chi(2))*[exp(lnc2)^(1-1/chi(2))+b(2)*[Le-exp(lnl2)]^(1-1/chi(2))]^[(1-1/gamma(2))/(1-1/chi(2))-1]]-exp(lnmu);
f8=lnap2-rho*lna2;

f9=exp(lnmu)*[1+phi*(exp(lnkp3)/exp(lnk3)-1)]-...
    beta*exp(lnmup)*[exp(lnthetap3)+exp(lnap3)*alpha*A*...
    exp(lnkp3)^(mupar(3)-1)*[alpha*exp(lnkp3)^(mupar(3))+(1-alpha)*exp(lnlp3)^(mupar(3))]^(1/mupar(3)-1)];
f9b=b(3)*exp(lnc3)^(1/chi(3))*(Le-exp(lnl3))^(-1/chi(3))-...
    exp(lna3)*(1-alpha)*A*exp(lnl3)^(mupar(3)-1)*[alpha*exp(lnk3)^(mupar(3))+(1-alpha)*exp(lnl3)^(mupar(3))]^(1/mupar(3)-1);
f10=exp(lntheta3)-[1+phi*[1+0.5*(exp(lnkp3)/exp(lnk3)-1)]*(exp(lnkp3)/exp(lnk3)-1)];
f11=[tau(3)*exp(lnc3)^(-1/chi(3))*[exp(lnc3)^(1-1/chi(3))+b(3)*[Le-exp(lnl3)]^(1-1/chi(3))]^[(1-1/gamma(3))/(1-1/chi(3))-1]]-exp(lnmu);
f12=lnap3-rho*lna3;

f13=exp(lnmu)*[1+phi*(exp(lnkp4)/exp(lnk4)-1)]-...
    beta*exp(lnmup)*[exp(lnthetap4)+exp(lnap4)*alpha*A*...
    exp(lnkp4)^(mupar(4)-1)*[alpha*exp(lnkp4)^(mupar(4))+(1-alpha)*exp(lnlp4)^(mupar(4))]^(1/mupar(4)-1)];
f13b=b(4)*exp(lnc4)^(1/chi(4))*(Le-exp(lnl4))^(-1/chi(4))-...
    exp(lna4)*(1-alpha)*A*exp(lnl4)^(mupar(4)-1)*[alpha*exp(lnk4)^(mupar(4))+(1-alpha)*exp(lnl4)^(mupar(4))]^(1/mupar(4)-1);
f14=exp(lntheta4)-[1+phi*[1+0.5*(exp(lnkp4)/exp(lnk4)-1)]*(exp(lnkp4)/exp(lnk4)-1)];
f15=[tau(4)*exp(lnc4)^(-1/chi(4))*[exp(lnc4)^(1-1/chi(4))+b(4)*[Le-exp(lnl4)]^(1-1/chi(4))]^[(1-1/gamma(4))/(1-1/chi(4))-1]]-exp(lnmu);
f16=lnap4-rho*lna4;

f17=exp(lnmu)*[1+phi*(exp(lnkp5)/exp(lnk5)-1)]-...
    beta*exp(lnmup)*[exp(lnthetap5)+exp(lnap5)*alpha*A*...
    exp(lnkp5)^(mupar(5)-1)*[alpha*exp(lnkp5)^(mupar(5))+(1-alpha)*exp(lnlp5)^(mupar(5))]^(1/mupar(5)-1)];
f17b=b(5)*exp(lnc5)^(1/chi(5))*(Le-exp(lnl5))^(-1/chi(5))-...
    exp(lna5)*(1-alpha)*A*exp(lnl5)^(mupar(5)-1)*[alpha*exp(lnk5)^(mupar(5))+(1-alpha)*exp(lnl5)^(mupar(5))]^(1/mupar(5)-1);
f18=exp(lntheta5)-[1+phi*[1+0.5*(exp(lnkp5)/exp(lnk5)-1)]*(exp(lnkp5)/exp(lnk5)-1)];
f19=[tau(5)*exp(lnc5)^(-1/chi(5))*[exp(lnc5)^(1-1/chi(5))+b(5)*[Le-exp(lnl5)]^(1-1/chi(5))]^[(1-1/gamma(5))/(1-1/chi(5))-1]]-exp(lnmu);
f20=lnap5-rho*lna5;

f21=exp(lnmu)*[1+phi*(exp(lnkp6)/exp(lnk6)-1)]-...
    beta*exp(lnmup)*[exp(lnthetap6)+exp(lnap6)*alpha*A*...
    exp(lnkp6)^(mupar(6)-1)*[alpha*exp(lnkp6)^(mupar(6))+(1-alpha)*exp(lnlp6)^(mupar(6))]^(1/mupar(6)-1)];
f21b=b(6)*exp(lnc6)^(1/chi(6))*(Le-exp(lnl6))^(-1/chi(6))-...
    exp(lna6)*(1-alpha)*A*exp(lnl6)^(mupar(6)-1)*[alpha*exp(lnk6)^(mupar(6))+(1-alpha)*exp(lnl6)^(mupar(6))]^(1/mupar(6)-1);
f22=exp(lntheta6)-[1+phi*[1+0.5*(exp(lnkp6)/exp(lnk6)-1)]*(exp(lnkp6)/exp(lnk6)-1)];
f23=[tau(6)*exp(lnc6)^(-1/chi(6))*[exp(lnc6)^(1-1/chi(6))+b(6)*[Le-exp(lnl6)]^(1-1/chi(6))]^[(1-1/gamma(6))/(1-1/chi(6))-1]]-exp(lnmu);
f24=lnap6-rho*lna6;

f25=exp(lnc1)+exp(lnc2)+exp(lnc3)+exp(lnc4)+exp(lnc5)+exp(lnc6)-exp(lnC);
f26=exp(lnk1)+exp(lnk2)+exp(lnk3)+exp(lnk4)+exp(lnk5)+exp(lnk6)-exp(lnK);

f27=exp(lna1)*A*[alpha*exp(lnk1)^(mupar(1))+(1-alpha)*exp(lnl1)^(mupar(1))]^(1/mupar(1))+...
    exp(lna2)*A*[alpha*exp(lnk2)^(mupar(2))+(1-alpha)*exp(lnl2)^(mupar(2))]^(1/mupar(2))+...
    exp(lna3)*A*[alpha*exp(lnk3)^(mupar(3))+(1-alpha)*exp(lnl3)^(mupar(3))]^(1/mupar(3))+...
    exp(lna4)*A*[alpha*exp(lnk4)^(mupar(4))+(1-alpha)*exp(lnl4)^(mupar(4))]^(1/mupar(4))+...
    exp(lna5)*A*[alpha*exp(lnk5)^(mupar(5))+(1-alpha)*exp(lnl5)^(mupar(5))]^(1/mupar(5))+...
    exp(lna6)*A*[alpha*exp(lnk6)^(mupar(6))+(1-alpha)*exp(lnl6)^(mupar(6))]^(1/mupar(6))-exp(lnY);

f28=exp(lnC)+exp(lnKp)-exp(lnK)-exp(lnY)+...
    phi/2*[exp(lnk1)*(exp(lnkp1)/exp(lnk1)-1)^2+exp(lnk2)*(exp(lnkp2)/exp(lnk2)-1)^2+...
    exp(lnk3)*(exp(lnkp3)/exp(lnk3)-1)^2+exp(lnk4)*(exp(lnkp4)/exp(lnk4)-1)^2+...
    exp(lnk5)*(exp(lnkp5)/exp(lnk5)-1)^2+exp(lnk6)*(exp(lnkp6)/exp(lnk6)-1)^2];

f=[f1;f1b;f2;f3;f4;f5;f5b;f6;f7;f8;f9;f9b;f10;f11;f12;f13;f13b;f14;f15;f16;f17;f17b;f18;f19;f20;f21;f21b;f22;f23;f24;f25;f26;f27;f28];

% 3) Define vectors of states and controls
xx=[lnk1 lnk2 lnk3 lnk4 lnk5 lnk6 lna1 lna2 lna3 lna4 lna5 lna6];
xxp=[lnkp1 lnkp2 lnkp3 lnkp4 lnkp5 lnkp6 lnap1 lnap2 lnap3 lnap4 lnap5 lnap6];

yy=[lnc1 lnc2 lnc3 lnc4 lnc5 lnc6 lntheta1 lntheta2 lntheta3 lntheta4 lntheta5 lntheta6 lnmu lnC lnK lnY lnl1 lnl2 lnl3 lnl4 lnl5 lnl6];
yyp=[lncp1 lncp2 lncp3 lncp4 lncp5 lncp6 lnthetap1 lnthetap2 lnthetap3 lnthetap4 lnthetap5 lnthetap6 lnmup lnCp lnKp lnYp lnlp1 lnlp2 lnlp3 lnlp4 lnlp5 lnlp6];

% 4) Compute analytical derivatives and evaluate them at the steady state
nx = length(xx);
ny = length(yy);

[fx,fxp,fy,fyp,fypyp,fypy,fypxp,fypx,fyyp,fyy,fyxp,fyx,fxpyp,fxpy,...
    fxpxp,fxpx,fxyp,fxy,fxxp,fxx]=anal_deriv(f,xx,yy,xxp,yyp);

assign_steadystate;
num_eval;

% 5) Construct log-linear approximation using the Klein (2000) algorithm
[gx,hx] = gx_hx(nfy,nfx,nfyp,nfxp);

if approx==2; 
etamat=zeros(nx,N+1);
for jj=1:N
    etamat(nx/2+jj,1)=sigma;
    etamat(nx/2+jj,1+jj)=sigma;
end
[GXX,HXX] = gxx_hxx(nfx,nfxp,nfy,nfyp,nfypyp,nfypy,nfypxp,nfypx,nfyyp,...
    nfyy,nfyxp,nfyx,nfxpyp,nfxpy,nfxpxp,nfxpx,nfxyp,nfxy,nfxxp,nfxx,hx,gx);
[GSS,HSS] = gss_hss(nfx,nfxp,nfy,nfyp,nfypyp,nfypy,nfypxp,nfypx,nfyyp,...
    nfyy,nfyxp,nfyx,nfxpyp,nfxpy,nfxpxp,nfxpx,nfxyp,nfxy,nfxxp,nfxx,hx,gx,GXX,etamat);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 3. Construct initial guess for decision rule coefficients %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% 2) Construct monomial grid (Integration formula Cn:5.2 in Stroud (1971))
nx=12;
r_Mon=sqrt(3/5);
V_MON=2^nx;
B0_MON=(25*nx^2-115*nx+162)/162*V_MON;
B1_MON=(70-25*nx)/162*V_MON;
B2_MON=25/324*V_MON;

% build grid	
xgrid=zeros(2*nx^2+1,nx);
for j=1:nx
xgrid(2*j,j)=r_Mon;
xgrid(2*j+1,j)=-r_Mon;
end

ind=1+2*nx+1;
interim=[r_Mon r_Mon;r_Mon -r_Mon;-r_Mon r_Mon;-r_Mon -r_Mon];

for j=1:(nx-1)
    for i=j+1:nx
        xgrid(ind:ind+3,j)=interim(:,1);
        xgrid(ind:ind+3,i)=interim(:,2);
        ind=ind+4;
    end
end

% Construct Chebyshev polynomials from grid
Cheby_k1=chebpol_2(ord,xgrid(:,1));
Cheby_k2=chebpol_2(ord,xgrid(:,2));
Cheby_k3=chebpol_2(ord,xgrid(:,3));
Cheby_k4=chebpol_2(ord,xgrid(:,4));
Cheby_k5=chebpol_2(ord,xgrid(:,5));
Cheby_k6=chebpol_2(ord,xgrid(:,6));

Cheby_a1=chebpol_2(ord,xgrid(:,7));
Cheby_a2=chebpol_2(ord,xgrid(:,8));
Cheby_a3=chebpol_2(ord,xgrid(:,9));
Cheby_a4=chebpol_2(ord,xgrid(:,10));
Cheby_a5=chebpol_2(ord,xgrid(:,11));
Cheby_a6=chebpol_2(ord,xgrid(:,12));

% Construct state vectors from grid
k1_vec=scalup(xgrid(:,1),kmin,kmax);
k2_vec=scalup(xgrid(:,2),kmin,kmax);
k3_vec=scalup(xgrid(:,3),kmin,kmax);
k4_vec=scalup(xgrid(:,4),kmin,kmax);
k5_vec=scalup(xgrid(:,5),kmin,kmax);
k6_vec=scalup(xgrid(:,6),kmin,kmax);
K_vec=k1_vec+k2_vec+k3_vec+k4_vec+k5_vec+k6_vec;

a1_vec=scalup(xgrid(:,7),amin,amax);
a2_vec=scalup(xgrid(:,8),amin,amax);
a3_vec=scalup(xgrid(:,9),amin,amax);
a4_vec=scalup(xgrid(:,10),amin,amax);
a5_vec=scalup(xgrid(:,11),amin,amax);
a6_vec=scalup(xgrid(:,12),amin,amax);

% Repeat the state vectors 14 times
k1_mat=kron(ones(14,1),k1_vec);
k2_mat=kron(ones(14,1),k2_vec);
k3_mat=kron(ones(14,1),k3_vec);
k4_mat=kron(ones(14,1),k4_vec);
k5_mat=kron(ones(14,1),k5_vec);
k6_mat=kron(ones(14,1),k6_vec);
K_mat=kron(ones(14,1),K_vec);

a1_mat=kron(ones(14,1),a1_vec);
a2_mat=kron(ones(14,1),a2_vec);
a3_mat=kron(ones(14,1),a3_vec);
a4_mat=kron(ones(14,1),a4_vec);
a5_mat=kron(ones(14,1),a5_vec);
a6_mat=kron(ones(14,1),a6_vec);

% Construct regressor matrix to be used for perturbation initialization
INIT_REG=[log(k1_vec)-log(kss) log(k2_vec)-log(kss) log(k3_vec)-log(kss) log(k4_vec)-log(kss) log(k5_vec)-log(kss) log(k6_vec)-log(kss) ...
    log(a1_vec)-log(ass) log(a2_vec)-log(ass) log(a3_vec)-log(ass) log(a4_vec)-log(ass) log(a5_vec)-log(ass) log(a6_vec)-log(ass)];

% Construct next period's capital stocks and current consumption
lnkp1_vec=log(kss)+INIT_REG*hx(1,:)';
lnkp2_vec=log(kss)+INIT_REG*hx(2,:)';
lnkp3_vec=log(kss)+INIT_REG*hx(3,:)';
lnkp4_vec=log(kss)+INIT_REG*hx(4,:)';
lnkp5_vec=log(kss)+INIT_REG*hx(5,:)';
lnkp6_vec=log(kss)+INIT_REG*hx(6,:)';
lnmu_vec=lnmuss+INIT_REG*gx(13,:)';

if approx==2; 
for i=1:size(INIT_REG,1);  
    lnkp1_vec(i,1)=lnkp1_vec(i,1)+0.5*[INIT_REG(i,:)*squeeze(HXX(1,:,:))*INIT_REG(i,:)'+HSS(1)];
    lnkp2_vec(i,1)=lnkp2_vec(i,1)'+0.5*[INIT_REG(i,:)*squeeze(HXX(2,:,:))*INIT_REG(i,:)'+HSS(2)];
    lnkp3_vec(i,1)=lnkp3_vec(i,1)+0.5*[INIT_REG(i,:)*squeeze(HXX(3,:,:))*INIT_REG(i,:)'+HSS(3)];
    lnkp4_vec(i,1)=lnkp4_vec(i,1)'+0.5*[INIT_REG(i,:)*squeeze(HXX(4,:,:))*INIT_REG(i,:)'+HSS(4)];
    lnkp5_vec(i,1)=lnkp5_vec(i,1)+0.5*[INIT_REG(i,:)*squeeze(HXX(5,:,:))*INIT_REG(i,:)'+HSS(5)];
    lnkp6_vec(i,1)=lnkp6_vec(i,1)'+0.5*[INIT_REG(i,:)*squeeze(HXX(6,:,:))*INIT_REG(i,:)'+HSS(6)];
    lnmu_vec(i,1)=lnmu_vec(i,1)'+0.5*[INIT_REG(i,:)*squeeze(GXX(13,:,:))*INIT_REG(i,:)'+GSS(13)];
end
end

% Compute initial guess for coefficient vectors by OLS
REG=makepoly_2_12(Cheby_k1,Cheby_k2,Cheby_k3,Cheby_k4,Cheby_k5,Cheby_k6,...
    Cheby_a1,Cheby_a2,Cheby_a3,Cheby_a4,Cheby_a5,Cheby_a6);

dr_k1_coeff0=inv(REG'*REG)*REG'*exp(lnkp1_vec);
dr_k2_coeff0=inv(REG'*REG)*REG'*exp(lnkp2_vec);
dr_k3_coeff0=inv(REG'*REG)*REG'*exp(lnkp3_vec);
dr_k4_coeff0=inv(REG'*REG)*REG'*exp(lnkp4_vec);
dr_k5_coeff0=inv(REG'*REG)*REG'*exp(lnkp5_vec);
dr_k6_coeff0=inv(REG'*REG)*REG'*exp(lnkp6_vec);

dr_mu_coeff0=inv(REG'*REG)*REG'*(lnmu_vec);


dr_coeff0=[dr_k1_coeff0;dr_k2_coeff0;dr_k3_coeff0;...
    dr_k4_coeff0;dr_k5_coeff0;dr_k6_coeff0;dr_mu_coeff0];


% Construct nodes to evaluate expectations in projection conditions
nodes=[zeros(14,7)];
nodes(1,1)=sqrt(2)*sqrt(7/2);
nodes(2,1)=-sqrt(2)*sqrt(7/2);
nodes(3,2)=sqrt(2)*sqrt(7/2);
nodes(4,2)=-sqrt(2)*sqrt(7/2);
nodes(5,3)=sqrt(2)*sqrt(7/2);
nodes(6,3)=-sqrt(2)*sqrt(7/2);
nodes(7,4)=sqrt(2)*sqrt(7/2);
nodes(8,4)=-sqrt(2)*sqrt(7/2);
nodes(9,5)=sqrt(2)*sqrt(7/2);
nodes(10,5)=-sqrt(2)*sqrt(7/2);
nodes(11,6)=sqrt(2)*sqrt(7/2);
nodes(12,6)=-sqrt(2)*sqrt(7/2);
nodes(13,7)=sqrt(2)*sqrt(7/2);
nodes(14,7)=-sqrt(2)*sqrt(7/2);

% Compute initial guess for coefficient vectors by OLS

% Repeat the period t regressors (N+1)*2 times
REG_mat=kron(ones(14,1),REG);

% Construct matrices for next periods exogenous states (outside loop saves
% time)
lnap1_mat=kron(ones(14,1),rho*log(a1_vec))+sigma*kron(nodes*[1;1;0;0;0;0;0],ones(size(REG,1),1));
lnap2_mat=kron(ones(14,1),rho*log(a2_vec))+sigma*kron(nodes*[1;0;1;0;0;0;0],ones(size(REG,1),1));
lnap3_mat=kron(ones(14,1),rho*log(a3_vec))+sigma*kron(nodes*[1;0;0;1;0;0;0],ones(size(REG,1),1));
lnap4_mat=kron(ones(14,1),rho*log(a4_vec))+sigma*kron(nodes*[1;0;0;0;1;0;0],ones(size(REG,1),1));
lnap5_mat=kron(ones(14,1),rho*log(a5_vec))+sigma*kron(nodes*[1;0;0;0;0;1;0],ones(size(REG,1),1));
lnap6_mat=kron(ones(14,1),rho*log(a6_vec))+sigma*kron(nodes*[1;0;0;0;0;0;1],ones(size(REG,1),1));

ap1_mat=exp(lnap1_mat);
ap2_mat=exp(lnap2_mat);
ap3_mat=exp(lnap3_mat);
ap4_mat=exp(lnap4_mat);
ap5_mat=exp(lnap5_mat);
ap6_mat=exp(lnap6_mat);

cheby_ap1_mat=chebpol_2(ord,scaldown(exp(lnap1_mat),amin,amax));
cheby_ap2_mat=chebpol_2(ord,scaldown(exp(lnap2_mat),amin,amax));
cheby_ap3_mat=chebpol_2(ord,scaldown(exp(lnap3_mat),amin,amax));
cheby_ap4_mat=chebpol_2(ord,scaldown(exp(lnap4_mat),amin,amax));
cheby_ap5_mat=chebpol_2(ord,scaldown(exp(lnap5_mat),amin,amax));
cheby_ap6_mat=chebpol_2(ord,scaldown(exp(lnap6_mat),amin,amax));

REG_A1=chebpol(8,2*[a1_mat-amin]/[amax-amin]-1);
REG_A2=chebpol(8,2*[a2_mat-amin]/[amax-amin]-1);
REG_A3=chebpol(8,2*[a3_mat-amin]/[amax-amin]-1);
REG_A4=chebpol(8,2*[a4_mat-amin]/[amax-amin]-1);
REG_A5=chebpol(8,2*[a5_mat-amin]/[amax-amin]-1);
REG_A6=chebpol(8,2*[a6_mat-amin]/[amax-amin]-1);

REG_K1=chebpol(8,2*[k1_mat-kmin]/[kmax-kmin]-1);
REG_K2=chebpol(8,2*[k2_mat-kmin]/[kmax-kmin]-1);
REG_K3=chebpol(8,2*[k3_mat-kmin]/[kmax-kmin]-1);
REG_K4=chebpol(8,2*[k4_mat-kmin]/[kmax-kmin]-1);
REG_K5=chebpol(8,2*[k5_mat-kmin]/[kmax-kmin]-1);
REG_K6=chebpol(8,2*[k6_mat-kmin]/[kmax-kmin]-1);

derive_labor_rules;
dr_coeff=dr_coeff0;

dr_coeff=fcsolve('projections_res',dr_coeff,[10000;1e-7;1e-8;1e-3;1]);
dr_coeff_A8N6=dr_coeff;
cpu1=cputime;
Time_Sol=cpu1-cpu0;
save('WSP_SOLUTION_A8_N6');
regvec_labor=[regvec_labor1 regvec_labor2 regvec_labor3 regvec_labor4 regvec_labor5 regvec_labor6];
save PI_A8N6.mat dr_coeff_A8N6 kmin kmax amin amax mumin mumax regvec_labor

